@@ -1,11 +1,11 @@
-Selection<-function(data,label=NULL, G=2, niter=50,method="GHD",starting="kmeans",max.iter=10,labelcl=NULL,q=2,scale=TRUE,criterion="ARI"){
+Selection<-function(data,label=NULL, G=2, niter=50,method="GHD",starting="kmeans",max.iter=10,eps=1e-2,labelcl=NULL,q=2,scale=TRUE,criterion="ARI"){
    if( criterion =="ARI"){
        if (is.null(label)) stop('label is null, please use criterion BIC ')
   ad=0
   mod=0
   if(method=="MGHFA"){
     for(i in 1:niter){
-      res=MGHFA(data,G=G,method=starting,label=labelcl,q=q,scale=scale)
+      res=MGHFA(data,G=G,method=starting,label=labelcl,max.iter=max.iter,q=q,scale=scale,eps=eps)
       adn=adjustedRandIndex(res$map,label)
       if(adn>ad){
         mod=res
@@ -14,7 +14,7 @@ Selection<-function(data,label=NULL, G=2, niter=50,method="GHD",starting="kmeans
     }}
   else if(method=="MSGHD"){
     for(i in 1:niter){
-      res=MSGHD(data,G=G,max.iter=max.iter,method=starting,label=labelcl,scale=scale)
+      res=MSGHD(data,G=G,max.iter=max.iter,method=starting,label=labelcl,scale=scale,eps=eps)
       adn=adjustedRandIndex(res$map,label)
       if(adn>ad){
         mod=res
@@ -22,7 +22,7 @@ Selection<-function(data,label=NULL, G=2, niter=50,method="GHD",starting="kmeans
     }}
   else if(method=="cMSGHD"){
       for(i in 1:niter){
-          res=cMSGHD(data,G=G,max.iter=max.iter,method=starting,label=labelcl,scale=scale)
+          res=cMSGHD(data,G=G,max.iter=max.iter,method=starting,label=labelcl,scale=scale,eps=eps)
           adn=adjustedRandIndex(res$map,label)
           if(adn>ad){
               mod=res
@@ -30,7 +30,7 @@ Selection<-function(data,label=NULL, G=2, niter=50,method="GHD",starting="kmeans
       }}
   else if(method=="MCGHD"){
     for(i in 1:niter){
-      res=MCGHD(data,G=G,max.iter=max.iter,method=starting,label=labelcl,scale=scale)
+      res=MCGHD(data,G=G,max.iter=max.iter,method=starting,label=labelcl,scale=scale,eps=eps)
       adn=adjustedRandIndex(res$map,label)
       if(adn>ad){
         mod=res
@@ -38,7 +38,7 @@ Selection<-function(data,label=NULL, G=2, niter=50,method="GHD",starting="kmeans
     }}
   else {
     for(i in 1:niter){
-      res=MGHD(data,G=G,n=max.iter,method=starting,label=labelcl,scale=scale)
+      res=MGHD(data,G=G,max.iter=max.iter,method=starting,label=labelcl,scale=scale,eps=eps)
       adn=adjustedRandIndex(res$map,label)
       if(adn>ad){
         mod=res
@@ -49,7 +49,7 @@ Selection<-function(data,label=NULL, G=2, niter=50,method="GHD",starting="kmeans
         BIC=-Inf
         if(method=="MGHFA"){
             for(i in 1:niter){
-                res=MGHFA(data,G=G,method=starting,label=labelcl,q=q,scale=scale)
+                res=MGHFA(data,G=G,method=starting,label=labelcl,max.iter=max.iter,q=q,scale=scale,eps=eps)
                 bicn=res$BIC
                 if(bicn>BIC){
                     mod=res
@@ -58,7 +58,7 @@ Selection<-function(data,label=NULL, G=2, niter=50,method="GHD",starting="kmeans
             }}
         else if(method=="MSGHD"){
             for(i in 1:niter){
-                res=MSGHD(data,G=G,max.iter=max.iter,method=starting,label=labelcl,scale=scale)
+                res=MSGHD(data,G=G,max.iter=max.iter,method=starting,label=labelcl,scale=scale,eps=eps)
                 bicn=res$BIC
                 if(bicn>BIC){
                     mod=res
@@ -66,7 +66,7 @@ Selection<-function(data,label=NULL, G=2, niter=50,method="GHD",starting="kmeans
             }}
         else if(method=="cMSGHD"){
             for(i in 1:niter){
-                res=cMSGHD(data,G=G,max.iter=max.iter,method=starting,label=labelcl,scale=scale)
+                res=cMSGHD(data,G=G,max.iter=max.iter,method=starting,label=labelcl,scale=scale,eps=eps)
                 bicn=res$BIC
                 if(bicn>BIC){
                     mod=res
@@ -74,14 +74,14 @@ Selection<-function(data,label=NULL, G=2, niter=50,method="GHD",starting="kmeans
             }}
         else if(method=="MCGHD"){
             for(i in 1:niter){
-                res=MCGHD(data,G=G,max.iter=max.iter,method=starting,label=labelcl,scale=scale)
+                res=MCGHD(data,G=G,max.iter=max.iter,method=starting,label=labelcl,scale=scale,eps=eps)
                 bicn=res$BIC
                 if(bicn>BIC){
                     mod=res
                     BIC=bicn}}}
         else {
             for(i in 1:niter){
-                res=MGHD(data,G=G,n=max.iter,method=starting,label=labelcl,scale=scale)
+                res=MGHD(data,G=G,max.iter=max.iter,method=starting,label=labelcl,scale=scale,eps=eps)
                 bicn=res$BIC
                 if(bicn>BIC){
                     mod=res
